{$mode objfpc}
{$h+}
{
    $Id: fplisttest.pp,v 1.5 2005/01/12 18:40:57 dean Exp $
    This file is part of the Free Component Library (FCL)
    Copyright (c) 2004, 2005 by Dean Zobec
    
    tests for TFPList and TFPObjectList 

    See the file COPYING.FPC, included in this distribution,
    for details about the copyright.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 **********************************************************************}
unit FPListtest;

interface

uses
  fpcunit, testregistry, classes, SysUtils;

type

  TFPListTestCase = class(TTestCase)
  private
    l: TFPList;
    obj: TObject;
    obj1: TObject;
  protected
    procedure Setup; override;
    procedure Teardown; override;
  published
    procedure CheckAdd;
    procedure TestExtract;
    procedure TestExtractNonExistent;
    procedure TestExtractNil;
    procedure TestGet;
    procedure TestGetNonExistent;
    procedure TestPut;
    procedure TestPutInWrongIndex;
    procedure TestPutNil;
    procedure TestInsertFirst;
    procedure TestInsertMiddle;
    procedure TestInsertLast;
    procedure TestSetCount;
    procedure TestSetSmallerCapacity;
    procedure TestClear;
    procedure TestDelete;
  end;

implementation

procedure TFPListTestCase.Setup;
begin
  l := TFPList.Create;
  obj := TObject.Create;
  obj1 := TObject.Create;
end;
    
procedure TFPListTestCase.Teardown; 
begin
  obj.Free;
  obj1.Free;
  l.Free;
end;

procedure TFPListTestCase.CheckAdd;
begin
  l.Add(obj);
  AssertEquals('Wrong number of items in FPList', 1, l.Count);
end;

procedure TFPListTestCase.TestExtract;
var
  extracted: TObject;
begin
  l.Add(obj);
  l.Add(obj1);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
  extracted := TObject(l.extract(obj));
  AssertSame('Wrong object extracted', extracted, obj);
  AssertEquals('Wrong number of items in FPList after extraction', 1, l.Count);
end;

procedure TFPListTestCase.TestExtractNonExistent;
var
  extracted: TObject;
begin
  l.Add(obj1);
  AssertEquals('Wrong number of items in FPList', 1, l.Count);
  extracted := TObject(l.extract(obj));
  AssertNull('Extracted should be nil', extracted);
  AssertEquals('Wrong number of items in FPList', 1, l.Count);
end;

procedure TFPListTestCase.TestExtractNil;
var
  extracted: TObject;
begin
  l.Add(obj);
  l.Add(obj1);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
  extracted := TObject(l.extract(nil));
  AssertNull('Extracted should be nil', extracted);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
end;

procedure TFPListTestCase.TestGet;
var
  examined: TObject;
begin
  l.Add(obj);
  l.Add(obj1);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
  examined:= TObject(l.items[1]);
  AssertSame('Got Wrong object', examined, obj1);
  examined:= TObject(l.items[0]);
  AssertSame('Got Wrong object', examined, obj);
end;

procedure TFPListTestCase.TestGetNonExistent;
begin
  l.Add(obj);
  l.Add(obj1);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
  try
    l.items[-1];
  except
    on E: EListError do
      Exit;
  end;
  Fail('No exception raised when getting a non existent item');
end;

procedure TFPListTestCase.TestPut;
begin
  l.Add(obj);
  l.Add(obj1);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
  l.Items[1] := obj;
  AssertSame('Error in putting object', obj, l.Items[1]);
end;

procedure TFPListTestCase.TestPutInWrongIndex;
begin
  l.Add(obj);
  l.Add(obj1);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
  try
    l.items[2] := obj;
  except
    on E: EListError do
      Exit;
  end;
  Fail('No exception raised when performin put out of range');
end;

procedure TFPListTestCase.TestPutNil;
begin
  l.Add(obj);
  AssertEquals('Wrong count after adding an item', 1, l.Count);
  l.Items[0] := nil;
  AssertNull('Error in setting an item to nil', l.Items[0]);
end;

procedure TFPListTestCase.TestInsertFirst;
begin
  l.add(obj);
  AssertEquals('Wrong number of items in FPList', 1, l.Count);
  l.Insert(0,obj1);
  AssertEquals('Wrong number of items after insert', 2, l.Count);
  AssertSame('Wrong item inserted', l[0], obj1);
  
end;

procedure TFPListTestCase.TestInsertMiddle;
begin
  l.add(obj);
  l.add(obj1);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
  l.Insert(1,obj1);
  AssertEquals('Wrong number of items after insert', 3, l.Count);
  AssertSame('Wrong item inserted', l[0], obj);
  AssertSame('Wrong item inserted', l[1], obj1);
  AssertSame('Wrong item inserted', l[2], obj1);
end;

procedure TFPListTestCase.TestInsertLast;
begin
  l.add(obj);
  l.add(obj1);
  AssertEquals('Wrong number of items in FPList', 2, l.Count);
  l.Insert(2,obj);
  AssertEquals('Wrong number of items after insert', 3, l.Count);
  AssertSame('Wrong item inserted', l[0], obj);
  AssertSame('Wrong item inserted', l[1], obj1);
  AssertSame('Wrong item inserted', l[2], obj);
end;

procedure TFPListTestCase.TestSetCount;
begin
  AssertEquals('wrong initial capacity', 0, l.capacity);
  AssertEquals('wrong initial count', 0, l.count);
  l.count := 1000;
  AssertEquals('wrong capacity', 1000, l.capacity);
  AssertEquals('wrong size', 1000, l.count);
  l[999] := nil;
  AssertNull('wrong item at count-1', l[l.count - 1]);
  AssertNull('item not nil', l[l.count -1 ]);
end;

procedure TFPListTestCase.TestSetSmallerCapacity;
var 
  i: integer;
begin
  for i := 0 to 100 do
    l.Add(obj);
  try
    l.capacity := 50;
  except
    on E: EListError do
      Exit;
  end;
  Fail('No exception raised when setting a capacity smaller than count');
end;

procedure TFPListTestCase.TestClear;
begin
  l.add(obj);
  l.add(obj1);
  AssertEquals('wrong number of items in FPList', 2, l.Count);
  l.Clear;
  AssertEquals('FPList was not cleared', 0, l.Count); 
end;

procedure TFPListTestCase.TestDelete;
begin
  l.add(obj);
  l.add(obj1);
  AssertEquals('wrong number of items in FPList', 2, l.Count);
  l.delete(0);
  AssertEquals('wrong number of items in FPList after delete', 1, l.Count);
  AssertEquals('object should not be in FPList, wrong indexOf', -1, l.IndexOf(obj));
end;

initialization
  
RegisterTests([TFPListTestCase]);


end.

