{$mode objfpc}
{$h+}
{
    $Id: fplisttest.pp,v 1.5 2005/01/12 18:40:57 dean Exp $
    This file is part of the Free Component Library (FCL)
    Copyright (c) 2004, 2005 by Michael Van Canneyt
    
    tests for TStringList

    See the file COPYING.FPC, included in this distribution,
    for details about the copyright.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 **********************************************************************}
unit StringListtest;

interface

uses
  fpcunit, testregistry, classes, SysUtils;

type

  { TStringListTestCase }

  TStringListTestCase = class(TTestCase)
  private
    l: TStringList;
    obj: TObject;
    obj1: TObject;
  protected
    procedure Setup; override;
    procedure Teardown; override;
  published
    procedure CheckAdd;
    procedure TestDelete;
    procedure TestDeleteNonExistent;
    procedure TestGetNonExistent;
    procedure TestGetExistent;
    procedure TestText;
    procedure TestPut;
    procedure TestPutInWrongIndex;
    procedure TestClear;
  end;

implementation

Const
  Line1 = 'Line1';
  Line2 = 'Line2';
  Line3 = 'Line3';

procedure TStringListTestCase.Setup;
begin
  l:=TStringList.Create;
  obj:=TObject.Create;
  obj1:=TObject.Create;
end;
    
procedure TStringListTestCase.Teardown; 
begin
  FreeAndNil(Obj);
  FreeAndNil(obj1);
  FreeAndNil(l);
end;

procedure TStringListTestCase.CheckAdd;
begin
  l.Add(Line1);
  AssertEquals('Wrong number of items in StringList', 1, l.Count);
  l.Add(Line2);
  AssertEquals('Wrong number of items in StringList', 2, l.Count);
  l.Add(Line3);
  AssertEquals('Wrong number of items in StringList', 3, l.Count);
end;

procedure TStringListTestCase.TestDelete;
var
  extracted: TObject;
begin
  l.Add(Line1);
  l.Add(Line2);
  L.Add(Line3);
  AssertEquals('Wrong number of items in StringList', l.Count,3);
  L.Delete(1);
  AssertEquals('Wrong number of items in StringList after delete', l.Count,2);
  AssertEquals('Wrong element deleted: before deleted element', l[1],Line3);
  AssertEquals('Wrong element deleted: after deleted element', l[0],Line1);
end;

procedure TStringListTestCase.TestDeleteNonExistent;

begin
  l.Add(Line1);
  AssertEquals('Wrong number of items in StringList', 1, l.Count);
  Try
    L.Delete(1);
  except
    Exit;
  end;
  Fail('No exception raised when deleting non-existent item');
end;

procedure TStringListTestCase.TestGetNonExistent;

var
  s : String;
  
begin
  l.Add(Line1);
  AssertEquals('Wrong number of items in StringList', 1, l.Count);
  Try
    S:=L[1];
  except
    Exit;
  end;
  Fail('No exception raised when retrieving non-existent item');
end;

procedure TStringListTestCase.TestGetExistent;
var
  S : String;
begin
  l.Add(Line1);
  AssertEquals('Wrong number of items in StringList', 1, l.Count);
  Try
    S:=L[0];
    AssertEquals('Wrong element returned !',S,L[0]);
  except
    Fail('Exception raised when retrieving existent item');
  end;
end;

procedure TStringListTestCase.TestText;

Var
  S : String;

begin
  l.Add(line1);
  l.Add(line2);
  L.add(line3);
  AssertEquals('Wrong number of items in StringList', 3, l.Count);
  S:=L.Text;
  AssertEquals('Text does not return correct value', Line1+LineEnding+Line2+LineEnding+Line3+LineEnding,S);
end;

procedure TStringListTestCase.TestPut;
begin
  l.Add(Line1);
  l.Add(Line2);
  AssertEquals('Wrong number of items in StringList', 2, l.Count);
  try
    l[1] := Line3;
  except
    Fail('Exception raised when assigning item in range');
  end;
  AssertEquals('Assigned item has wrong value', Line3, L[1]);
  AssertEquals('Other element modified by assign', Line1, L[0]);
end;

procedure TStringListTestCase.TestPutInWrongIndex;


begin
  l.Add(Line1);
  l.Add(Line2);
  AssertEquals('Wrong number of items in StringList', 2, l.Count);
  try
    l[2] := Line3;
  except
    on E: Exception do
      Exit;
  end;
  Fail('No exception raised when assigning  out of range');
end;



procedure TStringListTestCase.TestClear;
begin
  l.add(Line1);
  l.add(Line2);
  AssertEquals('wrong number of items in StringList', 2, l.Count);
  l.Clear;
  AssertEquals('StringList was not cleared', 0, l.Count); 
end;


initialization
  
RegisterTests([TStringListTestCase]);


end.

